/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
// histTool.cc: Main program for HistTool

#define DEBUG_SUMMARY 0
#define DEBUG_SYNCHRONIZE 0

// Define EXTERN to be blank so global variables will be allocated here
#define EXTERN

#define WELCOME "Welcome to HistTool"

#define TITLE_MAX 80

#define N_TEXT_ROWS 14
#define N_TEXT_ROWS_SERVER 32
#define N_TEXT_COLS 40

#define LIST_WIDTH 100
#define LIST_VISIBLE 10

#define GRAPH_HEIGHT 400
#define GRAPH_WIDTH 400

#define GIF_DIR_MASK "*.gif"

// mainCB
#define QUIT_BTN                 0
#define FILE_OPEN_BTN            1
#define FILE_NEW_BTN             2
#define FILE_SAVEAS_BTN          3
#define FILE_SAVE_BTN            4
#define CLEAR_BTN                7
#define PLOT_BTN                 8
#define NAME_ACTIVATE            9
#define FIRST_BTN                10
#define TOOLS_OPTIONS_BTN        16
#define OPTIONS_CANCEL_BTN       17

#include <time.h>
#include <sys/stat.h>
#include <errno.h>

#include "histTool.h"

#ifdef WIN32
/* WIN32 does not have unistd.h and does not define the following constants */
#define F_OK 00
#define W_OK 02
#define R_OK 04
#include <direct.h>     /* for getcwd (usually in sys/parm.h or unistd.h) */
#include <io.h>         /* for access, chmod  (usually in unistd.h) */
#else
/* Use unistd.h */
#include <unistd.h>
#endif

#include <X11/Xmu/Editres.h>
#include <X11/cursorfont.h>
#include <Xm/ArrowB.h>
#include <Xm/CascadeB.h>
#include <Xm/CascadeBG.h>
#include <Xm/DragDrop.h>
#include <Xm/DrawingA.h>
#include <Xm/FileSB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/MainW.h>
#include <Xm/MessageB.h>
#include <Xm/PanedW.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/SelectioB.h>
#include <Xm/Separator.h>
#include <Xm/Scale.h>
#include <Xm/ScrolledW.h>
#include <Xm/TextF.h>
#include <Xm/Text.h>
#include <Xm/ToggleB.h>

#include <Xm/MwmUtil.h>
#include <Xm/Protocols.h>
#include <X11/Xatom.h>

// Fallback resourses
#include "resources.h"

// Define help text
#include "histTool.hlp"

/* Structures */

/* Function prototypes */

int main(int argc, char **argv);
void createMainFormAreas(Widget wparent);
void helpCB(Widget, XtPointer clientData, XtPointer);
void mainCB(Widget w, XtPointer clientData, XtPointer callData);
void print(const char *fmt, ...);
static void quit(Widget w, XtPointer clientData, XtPointer callData);
void wmCloseCB(Widget w, XtPointer clientData, XtPointer callData);
int xerrMsg(const char *fmt, ...);
extern "C" int xerrorhandler(Display *dpy, XErrorEvent *event);
void xinfoMsg(char *sstring);
void xterrorhandler(char *message);
void dropProc(Widget w, XtPointer clientdata, XtPointer calldata);
static void yesNoCB(Widget w, XtPointer clientData, XtPointer callData);

#if 1
// Not implemented
void usage(void);
int parseCommand(int argc, char **argv);
#endif

#if 0
// Not used
void disposalMethodSet(int buttonId);
int setWindowTitle(char * fileName);
char *timeStamp(void);
#endif


/* Global variables */

char colorTableMergeModeString[4][25]={
    "Match Existing Colors",
    "Replace Unused Colors",
    "Add To Existing Colors",
    "Replace Unused, Then Add"
};

int nBlocks=0;
Widget appShell,mainMenu,mainForm,warningBox;
Widget helpS,helpST;
Widget yesNoD;
Widget optionsS;
Widget displayModeM,backgroundM,defaultColorTableM,colorTableMergeModeM;
Window rootWindow,appShellWindow,appMainWindowWindow,mainFormWindow;
Pixmap drawmap;
Atom WM_DELETE_WINDOW;
XtIntervalId playTimerID;
Boolean applyToAllImages;
Widget pvNameEntry=NULL;

GC gc;
XGCValues gcvalues;
Cursor watchCursor;
Visual *defvisual;
VisualID defvisualid;
int height,width,defplanes,defvisualclass;
int windowMessage=0;
char string[FIXED_MSG_SIZE];
char curText[FIXED_MSG_SIZE];
XmString cstring;
XColor sampleColor;
int fsdButton;

int colorTableMergeMode;
int defaultNColors,defaultNColorsBtn;

char startFileName[PATH_MAX];

void (*oldDropProc)(Widget w, XtPointer clientdata, XtPointer calldata);

/**************************** main ****************************************/
int main(int argc, char **argv)
{
    Widget w,w1,wparent,wparentsav;

  /* Set the error handler */
    errHandler=xerrMsg;

  /* WIN32 initialization */
#ifdef WIN32	
    HCLXmInit();
#endif

  /* Constants and defaults */
    yesNoD=optionsS=NULL;
    helpS=helpST=0;
    if(!options.read()) {
	errMsg("Could not read options initialization file");
    }

  /* Set up top level */
    XtToolkitInitialize();
    appContext=XtCreateApplicationContext();
    XtAppSetFallbackResources(appContext,fallbackResources);

  /* Get display parameters */
    display=XtOpenDisplay(appContext,NULL,"histTool","HistTool",NULL,0,&argc,argv);
    if(display == NULL) {
	xerrMsg("HistTool initialization: Cannot open display");
	exit(1);
    }
    screen=DefaultScreen(display);
    screenptr=ScreenOfDisplay(display,screen);
    nplanes=DisplayPlanes(display,screen);
    rootWindow=RootWindow(display,screen);
    defplanes=DefaultDepth(display,screen);
    defvisual=DefaultVisual(display,screen);
    defvisualid=XVisualIDFromVisual(defvisual);
    height=DisplayHeight(display,screen);
    width=DisplayWidth(display,screen);

  /* Parse the command line */
    if(parseCommand(argc,argv) != HT_OK) exit(1);

  /* Set error handlers */
    XtAppSetErrorHandler(appContext,xterrorhandler);
    XtAppSetWarningHandler(appContext,xterrorhandler);
    XSetErrorHandler(xerrorhandler);

  /* Define app shell */
    nargs=0;
    XtSetArg(args[nargs],XmNtitle,"Histogram Tool"); nargs++; 
    appShell=XtAppCreateShell("histTool","HistTool",applicationShellWidgetClass,
      display,args,nargs);

  /* Fix up WM close */
    WM_DELETE_WINDOW=XmInternAtom(display,"WM_DELETE_WINDOW",False);
    XmAddWMProtocolCallback(appShell,WM_DELETE_WINDOW,
      quit,(XtPointer)0);

  /* Enable Editres */
    XtAddEventHandler(appShell,(EventMask)NULL,TRUE,
      (XtEventHandler)_XEditResCheckMessages,(XtPointer)NULL);

#if DEBUG_SYNCHRONIZE
  /* Synchronize for debugging */
    XSynchronize(display,True);
#endif

  /* Define cursors */
    watchCursor=XCreateFontCursor(display,XC_watch);

  /* Define main window */
    nargs=0;
    appMainWindow=XmCreateMainWindow(appShell,"mainWindow",args,nargs);

    XtManageChild(appMainWindow);

  /* Define menu bar */
    nargs=0;
    mainMenu=XmCreateMenuBar(appMainWindow,"mainMenu",args,nargs);
    XtManageChild(mainMenu);
    wparent=mainMenu;

  /* Define file menu */
    nargs=0;
    XtSetArg(args[nargs],XmNtearOffModel,XmTEAR_OFF_ENABLED); nargs++;
    w1=XmCreatePulldownMenu(mainMenu,"fileMenu",args,nargs);

    nargs=0;
    XtSetArg(args[nargs],XmNsubMenuId,w1); nargs++;
    w=XmCreateCascadeButton(mainMenu,"File",args,nargs);
    XtManageChild(w);

#if 0
    nargs=0;
    w=XmCreatePushButton(w1,"New",args,nargs);
    XtManageChild(w);
    XtAddCallback(w,XmNactivateCallback,mainCB,(XtPointer)FILE_NEW_BTN);

    nargs=0;
    w=XmCreatePushButton(w1,"Open...",args,nargs);
    XtManageChild(w);
    XtAddCallback(w,XmNactivateCallback,mainCB,(XtPointer)FILE_OPEN_BTN);

    nargs=0;
    w=XmCreatePushButton(w1,"Save",args,nargs);
    XtManageChild(w);
    XtAddCallback(w,XmNactivateCallback,mainCB,(XtPointer)FILE_SAVE_BTN);

    nargs=0;
    w=XmCreatePushButton(w1,"Save As...",args,nargs);
    XtManageChild(w);
    XtAddCallback(w,XmNactivateCallback,mainCB,(XtPointer)FILE_SAVEAS_BTN);
#endif

    nargs=0;
    w=XmCreatePushButton(w1,"Quit",args,nargs);
    XtManageChild(w);
    XtAddCallback(w,XmNactivateCallback,mainCB,(XtPointer)QUIT_BTN);

  /* Define tools menu */
    nargs=0;
    XtSetArg(args[nargs],XmNtearOffModel,XmTEAR_OFF_ENABLED); nargs++;
    w1=XmCreatePulldownMenu(wparent,"toolsMenu",args,nargs);
    
    nargs=0;
    XtSetArg(args[nargs],XmNsubMenuId,w1); nargs++;
    w=XmCreateCascadeButton(mainMenu,"Tools",args,nargs);
    XtManageChild(w);

  /* Display Mode */
    wparentsav=wparent;
    wparent=w1;
        
  /* Options */
    nargs=0;
    w=XmCreatePushButton(wparent,"Options...",args,nargs);
    XtManageChild(w);
    XtAddCallback(w,XmNactivateCallback,mainCB,
      (XtPointer)TOOLS_OPTIONS_BTN);

  /* Define help menu */
    nargs=0;
    XtSetArg(args[nargs],XmNtearOffModel,XmTEAR_OFF_ENABLED); nargs++;
    w1=XmCreatePulldownMenu(mainMenu,"helpMenu",args,nargs);

    nargs=0;
    XtSetArg(args[nargs],XmNsubMenuId,w1); nargs++;
    w=XmCreateCascadeButton(mainMenu,"Help",args,nargs);
    XtManageChild(w);

    nargs=0;
    XtSetArg(args[nargs],XmNmenuHelpWidget,w); nargs++;
    XtSetValues(mainMenu,args,nargs);

    nargs=0;
    w=XmCreatePushButton(w1,"Overview",args,nargs);
    XtManageChild(w);
    XtAddCallback(w,XmNactivateCallback,helpCB,(XtPointer)0);

    nargs=0;
    w=XmCreatePushButton(w1,"Version",args,nargs);
    XtManageChild(w);
    XtAddCallback(w,XmNactivateCallback,helpCB,(XtPointer)1);

  /* Define form for rest of main window */
    nargs=0;
    mainForm=XmCreateForm(appMainWindow,"mainForm",args,nargs);
    XtManageChild(mainForm);

  /* Define text area */
    createMainFormAreas(mainForm);

  /* Set main window areas */
    XmMainWindowSetAreas(appMainWindow,mainMenu,NULL,NULL,NULL,mainForm);

  /* Realize */
    XtRealizeWidget(appShell);

  /* Get window ids */
    appShellWindow=XtWindow(appShell);
    appMainWindowWindow=XtWindow(appMainWindow);
    mainFormWindow=XtWindow(mainForm);

#if DEBUG_SUMMARY
    if(gif) gif->printSummary();
    exit(0);
#endif
    
  /* Start */
    windowMessage=1;
    XtAppMainLoop(appContext);
    return(0);
}
#if 0
/**************************** disposalMethodSet ***************************/
void disposalMethodSet(int buttonId)
{
    WidgetList buttons;
    Cardinal numButtons;
    Widget subMenu;

    XtVaGetValues(disposalMethodMenu,XmNsubMenuId,&subMenu,NULL);
    if (subMenu != NULL) {
	XtVaGetValues(subMenu, XmNchildren,&buttons, XmNnumChildren,
	  &numButtons, NULL);
	if (buttonId < (int)numButtons && buttonId >= 0) {
	    XtVaSetValues(disposalMethodMenu,XmNmenuHistory,buttons[buttonId],
	      NULL);
	}
    }
}
#endif
/**************************** dropProc ************************************/
void dropProc(Widget w, XtPointer clientdata, XtPointer calldata)
{
    XmTextPosition len;
    XmDropProcCallback dropData=(XmDropProcCallback)calldata;

  /* Delete the text that is there */    
    if(dropData->dropAction == XmDROP || dropData->dropAction == XmDROP_COPY) {
	len=XmTextGetLastPosition(w);
	XmTextSetSelection(w,0,len,dropData->timeStamp);
	XmTextRemove(w);
    }

#if 0
  /* Add a dragDropFinishCallback */
    XtAddCallback(dropdata->dragContext,XmNdragDropFinishCallback,
      dragdropfinishcb,(XtPointer)NULL);
    
  /* Add a workproc */
    XtAppAddWorkProc(appcontext,workproc,NULL);
    
  /* Add a timer proc (time in msec) */
    XtAppAddTimeOut(appcontext,200,timerproc,NULL);
#endif

  /* Call the old procedure */    
    (*oldDropProc)(w,clientdata,calldata);
}
/**************************** createMainFormAreas *************************/
void createMainFormAreas(Widget wparent)
{
    Widget w;
    Widget nameForm,controlForm;

  /* Create nameForm */
    nargs=0;
    XtSetArg(args[nargs],XmNtopAttachment,XmATTACH_FORM); nargs++;
    XtSetArg(args[nargs],XmNrightAttachment,XmATTACH_FORM); nargs++;
    XtSetArg(args[nargs],XmNleftAttachment,XmATTACH_FORM); nargs++;
    XtSetArg(args[nargs],XmNshadowThickness,0); nargs++;
    XtSetArg(args[nargs],XmNskipAdjust,False); nargs++;
    nameForm=XmCreateForm(wparent,"nameForm",args,nargs);

  /* Create PV entry */
    createLabelBox(nameForm,"labelBox","PV Name:",ECA_NAMESIZE,"",mainCB,
      (XtPointer)NAME_ACTIVATE,&pvNameEntry);

  /* Set initial name */
    XmTextSetString(pvNameEntry,START_PVNAME);

  /* Retrieve current drop site resources */
    nargs=0;
    XtSetArg(args[nargs],XmNdropProc,&oldDropProc); nargs++;
    XmDropSiteRetrieve(pvNameEntry,args,nargs);

  /* Update drop site resources */
    nargs=0;
    XtSetArg(args[nargs],XmNdropProc,dropProc); nargs++;
    XmDropSiteUpdate(pvNameEntry,args,nargs);

  /* Create control area */
    nargs=0;
    XtSetArg(args[nargs],XmNtopAttachment,XmATTACH_WIDGET); nargs++;
    XtSetArg(args[nargs],XmNtopWidget,nameForm); nargs++;
    XtSetArg(args[nargs],XmNrightAttachment,XmATTACH_FORM); nargs++;
    XtSetArg(args[nargs],XmNleftAttachment,XmATTACH_FORM); nargs++;
    XtSetArg(args[nargs],XmNbottomAttachment,XmATTACH_FORM); nargs++;
    XtSetArg(args[nargs],XmNshadowThickness,0); nargs++;
    XtSetArg(args[nargs],XmNfractionBase,5); nargs++;
    XtSetArg(args[nargs],XmNskipAdjust,True); nargs++;
    controlForm=XmCreateForm(wparent,"controlForm",args,nargs);

    nargs=0;
    XtSetArg(args[nargs],XmNtopAttachment,XmATTACH_FORM); nargs++;
    XtSetArg(args[nargs],XmNbottomAttachment,XmATTACH_FORM); nargs++;
    XtSetArg(args[nargs],XmNleftAttachment,XmATTACH_POSITION); nargs++;
    XtSetArg(args[nargs],XmNleftPosition,1); nargs++;
    XtSetArg(args[nargs],XmNrightAttachment,XmATTACH_POSITION); nargs++;
    XtSetArg(args[nargs],XmNrightPosition,2); nargs++;
    w=XmCreatePushButton(controlForm,"Clear",args,nargs);
    XtManageChild(w);
    XtAddCallback(w,XmNactivateCallback,mainCB,(XtPointer)CLEAR_BTN);
    
    nargs=0;
    XtSetArg(args[nargs],XmNtopAttachment,XmATTACH_FORM); nargs++;
    XtSetArg(args[nargs],XmNbottomAttachment,XmATTACH_FORM); nargs++;
    XtSetArg(args[nargs],XmNleftAttachment,XmATTACH_POSITION); nargs++;
    XtSetArg(args[nargs],XmNleftPosition,3); nargs++;
    XtSetArg(args[nargs],XmNrightAttachment,XmATTACH_POSITION); nargs++;
    XtSetArg(args[nargs],XmNrightPosition,4); nargs++;
    w=XmCreatePushButton(controlForm,"Plot",args,nargs);
    XtManageChild(w);
    XtAddCallback(w,XmNactivateCallback,mainCB,(XtPointer)PLOT_BTN);

  /* Manage the managers last */
    XtManageChild(nameForm);
    XtManageChild(controlForm);
}
/**************************** helpCB **************************************/
void helpCB(Widget w, XtPointer clientData, XtPointer callData)
{
    Widget w1,wparent,form;
    int i,n,nHelpStrings;
    n=(int)clientData;

    UNREFERENCED(w);
    UNREFERENCED(callData);

    switch(n) {
    case 0:
	if(!helpS) {
	    nargs=0;
	    XtSetArg(args[nargs],XmNtitle,"Help"); nargs++;
	    XtSetArg(args[nargs],XmNdeleteResponse, XmDO_NOTHING); nargs++;
	    helpS=XtCreatePopupShell("helpS",topLevelShellWidgetClass,
	      appShell,args,nargs);
	    XmAddWMProtocolCallback(helpS,WM_DELETE_WINDOW,
	      wmCloseCB,NULL);
	    
	    nargs=0;
	    form=XmCreateForm(helpS,"optPopup",args,nargs);
	    
	    nargs=0;
	    XtSetArg(args[nargs],XmNtopAttachment,XmATTACH_FORM); nargs++;
	    XtSetArg(args[nargs],XmNleftAttachment,XmATTACH_FORM); nargs++;
	    w1=XmCreatePushButton(form,"Remove",args,nargs);
	    XtManageChild(w1);
	    XtAddCallback(w1,XmNactivateCallback,helpCB,(XtPointer)2);
	    wparent=w1;
	    
	    nargs=0;
	    XtSetArg(args[nargs],XmNtopAttachment,XmATTACH_FORM); nargs++;
	    XtSetArg(args[nargs],XmNleftAttachment,XmATTACH_WIDGET); nargs++;
	    XtSetArg(args[nargs],XmNleftWidget,w1); nargs++;
	    w1=XmCreatePushButton(form,"Select All",args,nargs);
	    XtManageChild(w1);
	    XtAddCallback(w1,XmNactivateCallback,helpCB,(XtPointer)3);
	    
	    nargs=0;
	    XtSetArg(args[nargs],XmNtopAttachment,XmATTACH_WIDGET); nargs++;
	    XtSetArg(args[nargs],XmNtopWidget,wparent); nargs++;
	    XtSetArg(args[nargs],XmNleftAttachment,XmATTACH_FORM); nargs++;
	    XtSetArg(args[nargs],XmNrightAttachment,XmATTACH_FORM); nargs++;
	    XtSetArg(args[nargs],XmNbottomAttachment,XmATTACH_FORM); nargs++;
	    XtSetArg(args[nargs],XmNeditMode,XmMULTI_LINE_EDIT); nargs++;
	    XtSetArg(args[nargs],XmNeditable,False); nargs++;
	    XtSetArg(args[nargs],XmNcursorPositionVisible,False); nargs++;
	    XtSetArg(args[nargs],XmNrows,25); nargs++;
	    XtSetArg(args[nargs],XmNcolumns,80); nargs++;
	    w1=XmCreateScrolledText(form,"scrolledHelp",args,nargs);
	    helpST=w1;

	  // The help text must be broken down into string literals of
	  // length less than 2048 for WIN32.  Loop and set these strings
	  // into the text field.
	    nHelpStrings=sizeof(helptext)/sizeof(char *);
	    XmTextSetString(helpST,helptext[0]);
	    for(i=1; i < nHelpStrings; i++) {
		XmTextInsert(helpST,XmTextGetLastPosition(helpST),helptext[i]);
	    }
	    XtManageChild(helpST);
	    
	    XtManageChild(form);
	}

	if(helpS) XtPopup(helpS,XtGrabNone);
	break;
    case 1:
	sprintf(string,"%s\nCompilation Date: %s %s",HISTTOOL_VERSION_STRING,
	  __DATE__,__TIME__);
	xinfoMsg(string);
	break;
    case 2:
	if(helpS) XtPopdown(helpS);
	break;
    case 3:
	if(helpS) {
	    XmTextSetSelection(helpST,0,XmTextGetLastPosition(helpST),CurrentTime);
	    XmTextShowPosition(helpST,0);
	}
	break;
    }
}
/**************************** mainCB **************************************/
void mainCB(Widget w, XtPointer clientData, XtPointer callData)
{
    int id=(int)clientData;
    CHist *hist=NULL;
    char *name=NULL;

    switch(id) {
    case QUIT_BTN:
	quit(w,clientData,callData);
	break;
    case CLEAR_BTN:
	XmTextSetString(pvNameEntry,"");
	XmTextSetCursorPosition(pvNameEntry,0);
	break;
    case NAME_ACTIVATE:
    case PLOT_BTN:
	name=XmTextGetString(pvNameEntry);
	if(!name || !*name) {
	    XtFree(name);
	    XBell(display,50);
	    break;
	}
	hist=new CHist(name);
	if(hist) {
	    hist->setHelpCB(helpCB);
	    hist->setErrorHandler(xerrMsg);
	    XmTextSetCursorPosition(pvNameEntry,0);
	} else {
	    xerrMsg("Cannot create new window");
	}
	XtFree(name);
	break;
    case TOOLS_OPTIONS_BTN:
	CHist::createOptionsDialog(hist,True);
	break;
    case OPTIONS_CANCEL_BTN:
	XtPopdown(optionsS);
	break;
    }
}
/**************************** parseCommand ********************************/
int parseCommand(int argc, char **argv)
{
#if 1
    UNREFERENCED(argc);
    UNREFERENCED(argv);
#else
    int startFileSpecified=0;

    for(int i=1; i < argc; i++) {
	if (argv[i][0] == '-') {
	    switch(argv[i][1]) {
	    case 's':
		doServer=1;
		serverName=argv[++i];
		break;
	    case 'h':
		usage();
		return HT_OK;
	    default:
		fprintf(stderr,"\n\nInvalid option: %s\n",argv[i]);
		usage();
		return HT_ERROR;
	    }
	} else {
	    if(!startFileSpecified) {
		strcpy(startFileName,argv[i]);
		startFileSpecified=1;
	    } else {
		fprintf(stderr,"\n\nInvalid option: %s\n",argv[i]);
		usage();
		return HT_ERROR;
	    }
	}
    }
#endif
    return HT_OK;
}
/**************************** print ***************************************/
void print(const char *fmt, ...)
{
    va_list vargs;
    static char lstring[1024];  /* DANGER: Fixed buffer size */
    
    va_start(vargs,fmt);
    vsprintf(lstring,fmt,vargs);
    va_end(vargs);
    
    if(lstring[0] != '\0') {
#ifdef WIN32
	lprintf("%s",lstring);
#else
	printf("%s",lstring);
#endif
    }
}
/**************************** quit ****************************************/
static void quit(Widget w, XtPointer clientData, XtPointer callData)
{
    UNREFERENCED(w);
    UNREFERENCED(clientData);
    UNREFERENCED(callData);

    XtDestroyWidget(appShell);
    options.write();
    exit(0);
}
/**************************** setWindowTitle ******************************/
int setWindowTitle(char * fileName)
{
    char *ptr;
    char dirName[PATH_MAX];
    char name[TITLE_MAX];
    int dirLength;

    if(fileName) {
	strcpy(dirName,fileName);
	ptr=strrchr(dirName, DIR_DELIMITER_CHAR);
	if(ptr) {
	    *(++ptr)='\0';
	    dirLength=strlen(dirName);
	} else {
	    dirLength=0;
	}
	strcpy(name,"HistTool: ");
	strncpy(name+5,fileName+dirLength,TITLE_MAX-5);
	name[TITLE_MAX-1]='\0';
	nargs=0;
    } else {
	strcpy(name,"HistTool: New");
    }
    nargs=0;
    XtSetArg(args[nargs],XmNtitle,name); nargs++;
    XtSetValues(appShell,args,nargs);

    return dirLength;
}
/**************************** timestamp ***********************************/
char *timeStamp(void)
  // Gets current time and puts it in a static array
  // The calling program should copy it to a safe place
  //   e.g. strcpy(savetime,timestamp());
{
	static char timeStampStr[16];
	long now;
	struct tm *tblock;
	
	time(&now);
	tblock=localtime(&now);
	strftime(timeStampStr,20,"%b %d %H:%M:%S",tblock);
	
	return timeStampStr;
}
/**************************** usage ***************************************/
void usage(void)
{
#if 0
    printf(
      "\nUsage: perfstat [X Options] [Options]\n"
      "  Accumulates statistics for a performance database\n"
      "\n"
      "  Options:\n"
      "    -p prefix       Prefix for performance PVs [e.g. <prefix>perf:c0]\n"
      "                      Default is \"evans:\"\n"
      "    -s servername   Also do statistics for servername (e.g nike)\n"
      "    -h help         This message\n"
	);
#endif
}
/**************************** wmCloseCB ***********************************/
void wmCloseCB(Widget w, XtPointer clientData, XtPointer callData)
{
    UNREFERENCED(clientData);
    UNREFERENCED(callData);

    XtPopdown(w);
}
/**************************** xerrMsg *************************************/
int xerrMsg(const char *fmt, ...)
{
    Widget child;
    va_list vargs;
    static char lstring[FIXED_MSG_SIZE];
    
    va_start(vargs,fmt);
    (void)vsprintf(lstring,fmt,vargs);
    va_end(vargs);
    
    if(lstring[0] != '\0') {
	if(windowMessage) {
	    XBell(display,50); XBell(display,50); XBell(display,50); 
	    cstring=XmStringCreateLtoR(lstring,XmSTRING_DEFAULT_CHARSET);
	    nargs=0;
	    XtSetArg(args[nargs],XmNtitle,"Warning"); nargs++;
	    XtSetArg(args[nargs],XmNmessageString,cstring); nargs++;
	    warningBox=XmCreateWarningDialog(appMainWindow,"warningMessage",
	      args,nargs);
	    XmStringFree(cstring);
	    child=XmMessageBoxGetChild(warningBox,XmDIALOG_CANCEL_BUTTON);
	    XtDestroyWidget(child);
	    child=XmMessageBoxGetChild(warningBox,XmDIALOG_HELP_BUTTON);
	    XtDestroyWidget(child);
	    XtManageChild(warningBox);
	    XmUpdateDisplay(warningBox);
	}
#ifdef WIN32
	lprintf("%s\n",lstring);
#else
	fprintf(stderr,"%s\n",lstring);
#endif
    }

    return 0;
}
/**************************** xerrorhandler *******************************/
extern "C" int xerrorhandler(Display *dpy, XErrorEvent *event)
{
    char buf[FIXED_MSG_SIZE];

    XGetErrorText(dpy,event->error_code,buf,FIXED_MSG_SIZE);
    sprintf(string,"X Error:\n%s",buf);
#ifdef WIN32
    lprintf("%s\n",string);
#else
    fprintf(stderr,"%s\n",string);
#endif
    if(windowMessage) xerrMsg(string);
    return 0;
}
/**************************** xinfoMsg ************************************/
void xinfoMsg(char *sstring)
{
    Widget child,w;

    if(sstring[0] != '\0') {
	XmString sstringString=XmStringCreateLtoR(sstring,
	  XmSTRING_DEFAULT_CHARSET);
	XmString cancelString=XmStringCreateLocalized("Cancel");
	nargs=0;
	XtSetArg(args[nargs],XmNtitle,"Information"); nargs++;
	XtSetArg(args[nargs],XmNmessageString,sstringString); nargs++;
	XtSetArg(args[nargs],XmNokLabelString,cancelString); nargs++;
	w=XmCreateInformationDialog(appMainWindow,"informationMessage",args,nargs);
	XmStringFree(sstringString);
	XmStringFree(cancelString);
	child=XmMessageBoxGetChild(w,XmDIALOG_CANCEL_BUTTON);
	XtDestroyWidget(child);
	child=XmMessageBoxGetChild(w,XmDIALOG_HELP_BUTTON);
	XtDestroyWidget(child);
	XtManageChild(w);
    }
}
/**************************** xterrorhandler ******************************/
void xterrorhandler(char *message)
{
    static int nxtmessages=0;

#ifdef WIN32
    lprintf("%s\n",message);
#else
    fprintf(stderr,"%s\n",message);
#endif
    if(windowMessage) xerrMsg(string);
}
/**************************** yesNo ***************************************/
int yesNo(char *text, char *ok, char *cancel)
{
    static int response;
    XmString textString,okString,cancelString;
    
  // Create the yes/no dialog if necessary
    if(!yesNoD) {
	nargs=0;
	XtSetArg(args[nargs],XmNtitle,"Confirmation"); nargs++; 
	XtSetArg(args[nargs],XmNdialogStyle,XmDIALOG_FULL_APPLICATION_MODAL);
	  nargs++; 
	yesNoD=XmCreateQuestionDialog(appShell,"yesNoD",args,nargs);
	XtUnmanageChild(XmMessageBoxGetChild(yesNoD,XmDIALOG_HELP_BUTTON));
	XtAddCallback(yesNoD,XmNokCallback,yesNoCB,(XtPointer)&response);
	XtAddCallback(yesNoD,XmNcancelCallback,yesNoCB,(XtPointer)&response);
    }
    
  // Set the message and pop it up
    nargs=0;
    if(text) {
	textString=XmStringCreateLtoR(text,XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[nargs],XmNmessageString,textString); nargs++;
    }
    if(ok) {
	okString=XmStringCreateLtoR(ok,XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[nargs],XmNokLabelString,okString); nargs++;
    }
    if(cancel) {
	cancelString=XmStringCreateLtoR(cancel,XmSTRING_DEFAULT_CHARSET);
	XtSetArg(args[nargs],XmNcancelLabelString,cancelString); nargs++;
    }
    XtSetValues(yesNoD,args,nargs);
    if(text) XmStringFree(textString);
    if(ok) XmStringFree(okString);
    if(cancel) XmStringFree(cancelString);
    XtManageChild(yesNoD);
    XtPopup(XtParent(yesNoD),XtGrabNone);
    
  // Wait for a response
    response=0;
    while(!response) {
	XtAppProcessEvent(appContext,XtIMAll);
    }

    XtPopdown(XtParent(yesNoD));
    return response;
}
/**************************** yesNoCB *************************************/
static void yesNoCB(Widget w, XtPointer clientData, XtPointer callData)
{
    int *response=(int *)clientData;
    XmAnyCallbackStruct *cbs=(XmAnyCallbackStruct *)callData;

    UNREFERENCED(w);

    switch(cbs->reason) {
    case XmCR_OK:
	*response=1;
	break;
    case XmCR_CANCEL:
	*response=2;
	break;
    }
}
// optionMenuSet:  routine to set option menu to specified button index
//	(0 - (# buttons - 1))
//   Sets the XmNmenuHistory, which causes the button to be set
/**************************** optionMenuSet *******************************/
void optionMenuSet(Widget menu, int buttonId)
{
    WidgetList buttons;
    Cardinal numButtons;
    Widget subMenu;

  /* (MDA) - if option menus are ever created using non pushbutton or
   *	pushbutton widgets in them (e.g., separators) then this routine must
   *	loop over all children and make sure to only reference the push
   *	button derived children
   *
   *	Note: for invalid buttons, don't do anything (this can occur
   *	for example, when setting dynamic attributes when they don't
   *	really apply (and this is usually okay because they are not
   *	managed in invalid cases anyway))
   */
    XtVaGetValues(menu,XmNsubMenuId,&subMenu,NULL);
    if(subMenu != NULL) {
	XtVaGetValues(subMenu, XmNchildren,&buttons, XmNnumChildren,
	  &numButtons, NULL);
	if(buttonId < (int)numButtons && buttonId >= 0) {
	    XtVaSetValues(menu,XmNmenuHistory,buttons[buttonId],NULL);
	}
    } else {
	errMsg("optionMenuSet: No subMenu found for option menu");
    }
}
